package cn.com.qmth.stmms.ms.marking.api;

import cn.com.qmth.stmms.ms.core.domain.InspectRange;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.Paper;
import cn.com.qmth.stmms.ms.core.domain.user.MarkUser;
import cn.com.qmth.stmms.ms.core.repository.InspectRangeRepo;
import cn.com.qmth.stmms.ms.core.repository.LevelRepo;
import cn.com.qmth.stmms.ms.core.repository.MarkUserRepo;
import cn.com.qmth.stmms.ms.core.repository.PaperRepo;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.PaperAssembler;
import cn.com.qmth.stmms.ms.marking.dto.PaperDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

/**
 * Created by yuanpan on 2017/11/6.
 */
@RestController
@RequestMapping("api/inspect")
public class InspectApi {

    @Autowired
    private LevelRepo levelRepo;

    @Autowired
    private PaperRepo paperRepo;

    @Autowired
    private PaperAssembler paperAssembler;

    @Autowired
    private InspectRangeRepo inspectRangeRepo;

    @Autowired
    private MarkUserRepo markUserRepo;


    @GetMapping(value = "{workId}/{markerId}")
    public List<Map<String, Object>> list(@PathVariable(value = "workId") Long workId, @PathVariable(value = "markerId") Long markerId) {

        List<InspectRange> inspectRanges = inspectRangeRepo.findByWorkId(workId);
        if (inspectRanges == null || inspectRanges.size() < 1) {
            return null;
        }

        InspectRange inspectRange = inspectRanges.get(0);


        MarkUser inspector = markUserRepo.findOne(markerId);

        Subject subject = inspector.getSubject();


        List<PaperDTO> inspectPages = paperRepo.findByWorkIdAndSubjectAndInspectRange(workId, subject, inspectRange.getId()).stream().map(paper -> paperAssembler.toDTO(paper)).collect(Collectors.toList());
        Map<String, List<PaperDTO>> group = inspectPages.stream().collect(Collectors.groupingBy(PaperDTO::getAreaCode));

        List<Map<String, Object>> resultList = new LinkedList<>();
        group.forEach((k, v) -> {
            Map<String, Object> map = new HashMap<>();
            //将试卷按照分数大小从高到低排序
            v.sort((o1, o2) -> o2.getScore().intValue() - o1.getScore().intValue());
            map.put("area", k);
            map.put("papers", v);
            resultList.add(map);
        });

        //按数量倒序排序
        resultList.sort(new Comparator<Map<String, Object>>() {
            @Override
            public int compare(Map<String, Object> o1, Map<String, Object> o2) {
                List l1 = (List) o1.get("papers");
                List l2 = (List) o2.get("papers");

                return l2.size() - l1.size();
            }
        });
        return resultList;
    }


    @PostMapping(value = "{workId}/{markerId}/{paperId}")
    public void marking(@PathVariable(value = "workId") Long workId, @PathVariable(value = "markerId") Long markerId, @PathVariable(value = "paperId") Long paperId, @RequestBody HashMap<String, String> body) {

        Paper paper = paperRepo.findOne(paperId);
        if (Objects.equals(workId, paper.getWorkId())) {

            Double inspectScore = Double.parseDouble(body.get("score"));

            List<Level> levels = levelRepo.findByWorkId(workId);
            Level inspectLevel = null;

            for (Level l : levels) {
                if (l.getMaxScore() >= inspectScore && inspectScore >= l.getMinScore()) {
                    inspectLevel = l;
                    break;
                }
            }

            paper.setInspector(markerId);
            paper.setInspectScore(inspectScore);
            paper.setInspectLevel(inspectLevel.getCode());

            paperRepo.save(paper);

        }
    }

}
